<?php

class CheckingServerJob extends CronJob
{
    /**
     * Returns the name of the cronjob.
     */
    public static function getName()
    {
        return _('CheckStudip');
    }

    /**
     * Returns the description of the cronjob.
     */
    public static function getDescription()
    {
        return _('Fragt bei den registrierten Servern nach, ob es ihnen noch gut geht.');
    }

    /**
     * Setup method. Loads neccessary lib and checks environment. Will
     * bail out with an exception if environment does not match requirements.
     */
    public function setUp()
    {
        require_once __DIR__."/CheckStudip.class.php";
        require_once __DIR__."/lib/CheckRemoteServer.php";
        require_once __DIR__."/lib/CheckMonitoringData.php";
        require_once __DIR__."/lib/CheckMonitoringPoint.php";
        require_once __DIR__."/lib/CheckStudipMetric.php";
    }

    /**
     * Return the parameters for this cronjob.
     *
     * @return Array Parameters.
     */
    public static function getParameters()
    {
        return [];
    }

    /**
     * Executes the cronjob.
     *
     * @param mixed $last_result What the last execution of this cronjob
     *                           returned.
     * @param Array $parameters Parameters for this cronjob instance which
     *                          were defined during scheduling.
     *                          Only valid parameter at the moment is
     *                          "verbose" which toggles verbose output while
     *                          purging the cache.
     */
    public function execute($last_result, $parameters = array())
    {
        $already_exists = false;
        $point = new CheckMonitoringPoint();
        $point['planned_time'] = round(time() / 600) * 600;
        try {
            $point->store();
        } catch(Exception $e) {
            $already_exists = true;
        }
        if (!$already_exists) {
            $servers = CheckRemoteServer::findBySQL("1=1 ORDER BY name ASC");

            $use_multi_curl = false;

            if ($use_multi_curl) {
                $multi = curl_multi_init();
                $requests = [];
                foreach ($servers as $server) {
                    $requests[] = $server->requestData($multi);
                }

                do {
                    $status = curl_multi_exec($multi, $active);
                    if ($active) {
                        // Einen Moment auf weitere Aktivität warten
                        curl_multi_select($multi);
                    }
                } while ($active && $status == CURLM_OK);

                // Close the handles
                foreach ($requests as $r) {
                    $contents = curl_multi_getcontent($r['request']);
                    if ($contents) {
                        $contents = json_decode($contents, true);
                        $contents['success'] = true;
                    } else {
                        $error = curl_error($r['request']);
                        $contents = [
                            'success' => false,
                            'error' => $error
                        ];
                    }
                    $data = new CheckMonitoringData();
                    $data['checkpoint_id'] = $point->id;
                    $data['server_id'] = $r['server']->id;
                    $data['success'] = $contents['success'];
                    $data['request_time'] = null;
                    foreach (CheckStudip::getMetrics() as $metric) {
                        $data[$metric->getIndex()] = $output[$metric->getIndex()] ?? null;
                    }
                    $data->store();

                    echo ( $contents['success'] ? '✓' : '❌') . ' '. $r['server']->name . "\n";

                    if ($r['server']['expire_days'] > 0) {
                        CheckMonitoringData::deleteBySQL('server_id = :server_id AND mkdate < :expire', [
                            'server_id' => $r['server']->getId(),
                            'expire' => time() - $r['server']['expire_days'] * 86400
                        ]);
                    }
                    curl_multi_remove_handle($multi, $r['request']);
                }
                curl_multi_close($multi);
            } else {
                foreach ($servers as $server) {
                    $output = $server->fetch();

                    $data = new CheckMonitoringData();
                    $data['checkpoint_id'] = $point->id;
                    $data['server_id'] = $server->id;
                    $data['success'] = $output['success'];
                    $data['request_time'] = $output['request_time'] ?? null;
                    foreach (CheckStudip::getMetrics() as $metric) {
                        $data[$metric->getIndex()] = $output[$metric->getIndex()] ?? null;
                    }
                    $data->store();

                    echo ($data['success'] ? '✓' : '❌') . ' '. $server->name . "\n";

                    if ($server['expire_days'] > 0) {
                        CheckMonitoringData::deleteBySQL('server_id = :server_id AND mkdate < :expire', [
                            'server_id' => $server->getId(),
                            'expire' => time() - $server['expire_days'] * 86400
                        ]);
                    }
                }
            }
        }
    }
}
